#include "menu.h"
#include "handler.h"
#include "key.h"
#include <stdio.h>
#include <string.h>

cui
cui_menu_btn_new(cui parent, int x, int y, char *s)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_btn));
	cui_menu_btn_init(obj, parent, x, y, s);
	return obj;
}

void
cui_menu_btn_init(cui obj, cui parent, int x, int y, char *s)
{
	cui_button btn = (cui_button)obj;

	cui_button_init(obj, parent, x, y, s);
	cui_bind(obj, CUI_EVT_DRAW, cui_menu_btn_hdr, NULL);
	cui_hide(btn->lb1);
	cui_hide(obj);
	cui_label_str_set(btn->lb3, "|");
	btn->lb2->x -= 1;
	btn->lb3->x -= 1;
	obj->w = 0;
	cui_wh_fit(obj);
	cui_label_attr_set(btn->lb2, CUI_ATTR_ULINE);
	cui_label_attr_set(btn->lb3, CUI_ATTR_ULINE);
	cui_show(obj);
}

int
cui_menu_btn_hdr(cui obj, int evt, int val, void *prm)
{
	cui_button btn = (cui_button)obj;
	int attr = CUI_ATTR_ULINE;
	if(val == CUI_DRAW_FOCUS) attr |= CUI_ATTR_REVERSE;
	cui_label_attr_set(btn->lb2, attr);
	cui_label_attr_set(btn->lb3, attr);
	return TRUE;
}

char *
cui_menu_btn_str_get(cui obj)
{
	return cui_button_str_get(obj);
}

void
cui_menu_btn_str_set(cui obj, char *s)
{
	cui_button_str_set(obj, s);
}

cui
cui_menu_item_new(cui parent, int x, int y, char *s)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_item));
	cui_menu_item_init(obj, parent, x, y, s);
	return obj;
}

void
cui_menu_item_init(cui obj, cui parent, int x, int y, char *s)
{
	cui_menu_item p = (cui_menu_item)obj;

	cui_base_init(obj, parent, x, y, 0, 1);
	cui_hide(obj);
	p->lbf = cui_lblfix_new(obj, 0, 0, -1, s, CUI_ATTR_ULINE);
	p->lb_end = cui_label_new(obj, cui_x2(p->lbf), 0, "|");
	cui_label_attr_set(p->lb_end, CUI_ATTR_ULINE);
	cui_bind(obj, CUI_EVT_DRAW | CUI_EVT_KEY | CUI_EVT_RESIZE, cui_menu_item_hdr, NULL);
	cui_wh_fit(obj);
	cui_show(obj);
	obj->flags |= CUI_FLG_CAN_FOCUS;
}

int
cui_menu_item_hdr(cui obj, int evt, int val, void *prm)
{
	cui_menu_item p = (cui_menu_item)obj;
	int attr, dir;
	cui obj2;

	switch(evt){
	case CUI_EVT_BUTTON:
		val = val ? cui_index(obj) : -1;
		cui_handler_call(obj->parent, CUI_EVT_BUTTON, val);
		return TRUE;
	case CUI_EVT_KEY:
		switch(val){
		case CUI_KEY_ENTER:
		case CUI_KEY_ESC:
			cui_handler_call(obj, CUI_EVT_BUTTON, val == CUI_KEY_ENTER);
			return TRUE;
		case CUI_KEY_DOWN:
		case CUI_KEY_UP:
			dir = val == CUI_KEY_DOWN ? 1 : -1;
			obj2 = cui_index_to_child(obj->parent, cui_index(obj) + dir);
			if(obj2 && !cui_is_visible_rect(obj2)){
				cui_y_set(obj->parent, obj->parent->y - dir);
			}
			break;
		}
		return FALSE;
	case CUI_EVT_DRAW:
		attr = CUI_ATTR_ULINE;
		if(val == CUI_DRAW_FOCUS) attr |= CUI_ATTR_REVERSE;
		cui_lblfix_attr_set(p->lbf, attr);
		cui_label_attr_set(p->lb_end, attr);
		return TRUE;
	case CUI_EVT_RESIZE:
		cui_w_set(p->lbf, obj->w - 1);
		cui_x_set(p->lb_end, cui_x2(p->lbf));
		return TRUE;
	}
	return FALSE;
}

char *
cui_menu_item_str_get(cui obj)
{
	cui_menu_item p = (cui_menu_item)obj;
	return cui_lblfix_str_get(p->lbf);
}

void
cui_menu_item_str_set(cui obj, char *s)
{
	int bak = cui_hide_enter(obj);
	cui_menu_item p = (cui_menu_item)obj;
	cui_lblfix_str_set(p->lbf, s);
	cui_hide_exit(obj, bak);
}

cui
cui_menu_new(cui parent, int x, int y, char **lst)
{
	cui obj = cui_base_new(parent, x, y, 0, 0);
	char *s;
	cui item;
	
	cui_hide(obj);
	y = 0;
	while((s = *lst++) != NULL) cui_menu_item_new(obj, 0, y++, s);
	cui_wh_fit(obj);
	for(item=obj->children; item; item=item->next){	
		cui_w_set(item, obj->w);
		cui_bind(item, CUI_EVT_BUTTON, cui_menu_item_hdr, NULL);
	}
	return obj;
}

cui
cui_menu_str_to_item(cui menu, char *s)
{
	cui item;

	for(item=menu->children; item; item=item->next){
		if(strcmp(cui_menu_item_str_get(item), s) == 0) return item;
	}
	return NULL;
}

cui
cui_menu_popup_new(cui parent, int x, int y, int w, char **lst, int val)
{
	cui obj = cui_alloc(sizeof(struct cui_menu_popup));
	cui_menu_popup_init(obj, parent, x, y, w, lst, val);
	return obj;
}

void
cui_menu_popup_init(cui obj, cui parent, int x, int y, int w, char **lst, int val)
{
	cui_menu_popup p = (cui_menu_popup)obj;

	cui_menu_item_init(obj, parent, x, y, lst[val]);
	if(w >= 0) cui_w_set(obj, w);
	p->menu = cui_menu_new(cui_root(obj), 0, 0, lst);
	p->val = val;

	cui_unbind(obj, CUI_EVT_KEY | CUI_EVT_BUTTON, cui_menu_item_hdr);
	cui_bind(obj, CUI_EVT_KEY, cui_menu_popup_hdr, NULL);
	cui_bind(p->menu, CUI_EVT_BUTTON, cui_menu_popup_hdr, obj);
}

int
cui_menu_popup_hdr(cui obj, int evt, int val, void *prm)
{
	if(evt == CUI_EVT_KEY && val == CUI_KEY_ENTER){ /* menu_popup */
		cui_menu_popup p = (cui_menu_popup)obj;
		cui menu = p->menu;
		int bak = p->val;
		menu->x = cui_lx(cui_root(obj), cui_gx(obj));
		menu->y = cui_ly(cui_root(obj), cui_gy(obj)) - p->val;
		menu->flags &= ~CUI_FLG_HIDE;
		cui_main(menu, cui_index_to_child(menu, p->val));
		cui_hide(menu);
		if(p->val < 0) p->val = bak;
		else{
			char *s = cui_menu_item_str_get(cui_index_to_child(menu, p->val));
			cui_menu_item_str_set(obj, s);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->val);
		}
		cui_draw(menu->parent);
		return TRUE;
	}else if(evt == CUI_EVT_BUTTON){ /* menu */
		cui_menu_popup p = (cui_menu_popup)prm;
		p->val = val;
		cui_quit();
		return TRUE;
	}
	return FALSE;
}

char *
cui_menu_popup_str_get(cui obj)
{
	return cui_menu_item_str_get(obj);
}

/* EOF */
